@@ -155,11 +155,11 @@ class EventImpl : public Event {
   EventImpl (Type type, ErrorCode err, Severity severity,
              const char *fac, const char *str):
   type_(type), err_(err), severity_(severity), fac_(fac ? fac : ""),
-	  str_(str), id_(0), throttle_time_(0) {};
+  str_(str), id_(0), throttle_time_(0), fatal_(false) {};
 
   EventImpl (Type type):
   type_(type), err_(ERR_NO_ERROR), severity_(EVENT_SEVERITY_EMERG),
-	  fac_(""), str_(""), id_(0), throttle_time_(0) {};
+  fac_(""), str_(""), id_(0), throttle_time_(0), fatal_(false) {};
 
   Type        type () const { return type_; }
   ErrorCode   err () const { return err_; }
@@ -467,7 +467,7 @@ private:
 
 class ConfImpl : public Conf {
  public:
-  ConfImpl()
+  ConfImpl(ConfType conf_type)
       :consume_cb_(NULL),
       dr_cb_(NULL),
       event_cb_(NULL),
@@ -479,8 +479,10 @@ class ConfImpl : public Conf {
       offset_commit_cb_(NULL),
       oauthbearer_token_refresh_cb_(NULL),
       ssl_cert_verify_cb_(NULL),
+      conf_type_(conf_type),
       rk_conf_(NULL),
-      rkt_conf_(NULL){}
+      rkt_conf_(NULL)
+      {}
   ~ConfImpl () {
     if (rk_conf_)
       rd_kafka_conf_destroy(rk_conf_);
@@ -683,6 +685,18 @@ class ConfImpl : public Conf {
     return Conf::CONF_OK;
   }
 
+  Conf::ConfResult set_engine_callback_data (void *value,
+                                             std::string &errstr) {
+    if (!rk_conf_) {
+      errstr = "Requires RdKafka::Conf::CONF_GLOBAL object";
+      return Conf::CONF_INVALID;
+    }
+
+    rd_kafka_conf_set_engine_callback_data(rk_conf_, value);
+    return Conf::CONF_OK;
+  }
+
+
   Conf::ConfResult set_ssl_cert (RdKafka::CertificateType cert_type,
                                  RdKafka::CertificateEncoding cert_enc,
                                  const void *buffer, size_t size,
@@ -977,7 +991,7 @@ class HandleImpl : virtual public Handle {
                                                extensions_copy,
                                                extensions.size(),
                                                errbuf, sizeof(errbuf)));
-          free(extensions_copy);
+          delete[] extensions_copy;
 
           if (err != ERR_NO_ERROR)
               errstr = errbuf;
@@ -990,6 +1004,13 @@ class HandleImpl : virtual public Handle {
                                                 rk_, errstr.c_str()));
   };
 
+  void *mem_malloc (size_t size) {
+    return rd_kafka_mem_malloc(rk_, size);
+  };
+
+  void mem_free (void *ptr) {
+    rd_kafka_mem_free(rk_, ptr);
+  };
 
   rd_kafka_t *rk_;
   /* All Producer and Consumer callbacks must reside in HandleImpl and
@@ -1116,11 +1137,18 @@ public:
   static KafkaConsumer *create (Conf *conf, std::string &errstr);
 
   ErrorCode assignment (std::vector<TopicPartition*> &partitions);
+  bool assignment_lost ();
+  std::string rebalance_protocol () {
+    const char *str = rd_kafka_rebalance_protocol(rk_);
+    return std::string(str ? str : "");
+  }
   ErrorCode subscription (std::vector<std::string> &topics);
   ErrorCode subscribe (const std::vector<std::string> &topics);
   ErrorCode unsubscribe ();
   ErrorCode assign (const std::vector<TopicPartition*> &partitions);
   ErrorCode unassign ();
+  Error *incremental_assign (const std::vector<TopicPartition*> &partitions);
+  Error *incremental_unassign (const std::vector<TopicPartition*> &partitions);
 
   Message *consume (int timeout_ms);
   ErrorCode commitSync () {
