@@ -45,6 +45,8 @@ typedef struct rd_kafka_interceptor_method_s {
                 rd_kafka_interceptor_f_on_consume_t *on_consume;
                 rd_kafka_interceptor_f_on_commit_t  *on_commit;
                 rd_kafka_interceptor_f_on_request_sent_t *on_request_sent;
+                rd_kafka_interceptor_f_on_response_received_t
+                *on_response_received;
                 rd_kafka_interceptor_f_on_thread_start_t *on_thread_start;
                 rd_kafka_interceptor_f_on_thread_exit_t  *on_thread_exit;
                 void *generic; /* For easy assignment */
@@ -162,7 +164,7 @@ rd_kafka_interceptor_method_add (rd_list_t *list, const char *ic_name,
 
 /**
  * @brief Destroy all interceptors
- * @locality application thread calling rd_kafka_conf_destroy() or 
+ * @locality application thread calling rd_kafka_conf_destroy() or
  *           rd_kafka_destroy()
  */
 void rd_kafka_interceptors_destroy (rd_kafka_conf_t *conf) {
@@ -176,6 +178,7 @@ void rd_kafka_interceptors_destroy (rd_kafka_conf_t *conf) {
         rd_list_destroy(&conf->interceptors.on_consume);
         rd_list_destroy(&conf->interceptors.on_commit);
         rd_list_destroy(&conf->interceptors.on_request_sent);
+        rd_list_destroy(&conf->interceptors.on_response_received);
         rd_list_destroy(&conf->interceptors.on_thread_start);
         rd_list_destroy(&conf->interceptors.on_thread_exit);
 
@@ -220,6 +223,9 @@ rd_kafka_interceptors_init (rd_kafka_conf_t *conf) {
         rd_list_init(&conf->interceptors.on_request_sent, 0,
                      rd_kafka_interceptor_method_destroy)
                 ->rl_flags |= RD_LIST_F_UNIQUE;
+        rd_list_init(&conf->interceptors.on_response_received, 0,
+                     rd_kafka_interceptor_method_destroy)
+                ->rl_flags |= RD_LIST_F_UNIQUE;
         rd_list_init(&conf->interceptors.on_thread_start, 0,
                      rd_kafka_interceptor_method_destroy)
                 ->rl_flags |= RD_LIST_F_UNIQUE;
@@ -567,6 +573,46 @@ void rd_kafka_interceptors_on_request_sent (rd_kafka_t *rk,
 }
 
 
+/**
+ * @brief Call interceptor on_response_received methods
+ * @locality internal broker thread
+ */
+void rd_kafka_interceptors_on_response_received (rd_kafka_t *rk,
+                                                 int sockfd,
+                                                 const char *brokername,
+                                                 int32_t brokerid,
+                                                 int16_t ApiKey,
+                                                 int16_t ApiVersion,
+                                                 int32_t CorrId,
+                                                 size_t  size,
+                                                 int64_t rtt,
+                                                 rd_kafka_resp_err_t err) {
+        rd_kafka_interceptor_method_t *method;
+        int i;
+
+        RD_LIST_FOREACH(method, &rk->rk_conf.interceptors.on_response_received,
+                        i) {
+                rd_kafka_resp_err_t ic_err;
+
+                ic_err = method->u.on_response_received(rk,
+                                                        sockfd,
+                                                        brokername,
+                                                        brokerid,
+                                                        ApiKey,
+                                                        ApiVersion,
+                                                        CorrId,
+                                                        size,
+                                                        rtt,
+                                                        err,
+                                                        method->ic_opaque);
+                if (unlikely(ic_err))
+                        rd_kafka_interceptor_failed(rk, method,
+                                                    "on_response_received",
+                                                    ic_err, NULL, NULL);
+        }
+}
+
+
 void
 rd_kafka_interceptors_on_thread_start (rd_kafka_t *rk,
                                        rd_kafka_thread_type_t thread_type) {
@@ -733,6 +779,20 @@ rd_kafka_interceptor_add_on_request_sent (
 }
 
 
+rd_kafka_resp_err_t
+rd_kafka_interceptor_add_on_response_received (
+        rd_kafka_t *rk, const char *ic_name,
+        rd_kafka_interceptor_f_on_response_received_t *on_response_received,
+        void *ic_opaque) {
+        assert(!rk->rk_initialized);
+        return rd_kafka_interceptor_method_add(&rk->rk_conf.interceptors.
+                                               on_response_received,
+                                               ic_name,
+                                               (void *)on_response_received,
+                                               ic_opaque);
+}
+
+
 rd_kafka_resp_err_t
 rd_kafka_interceptor_add_on_thread_start (
         rd_kafka_t *rk, const char *ic_name,
