@@ -194,7 +194,7 @@ int rd_kafka_metadata_partition_id_cmp (const void *_a,
  *
  * This makes a copy of \p topic
  *
- * @locks rd_kafka_wrlock()
+ * @locks_required rd_kafka_wrlock()
  */
 static struct rd_kafka_metadata_cache_entry *
 rd_kafka_metadata_cache_insert (rd_kafka_t *rk,
@@ -267,9 +267,9 @@ rd_kafka_metadata_cache_insert (rd_kafka_t *rk,
 /**
  * @brief Purge the metadata cache
  *
- * @locks rd_kafka_wrlock()
+ * @locks_required rd_kafka_wrlock()
  */
-static void rd_kafka_metadata_cache_purge (rd_kafka_t *rk) {
+void rd_kafka_metadata_cache_purge (rd_kafka_t *rk, rd_bool_t purge_observers) {
         struct rd_kafka_metadata_cache_entry *rkmce;
         int was_empty = TAILQ_EMPTY(&rk->rk_metadata_cache.rkmc_expiry);
 
@@ -281,6 +281,9 @@ static void rd_kafka_metadata_cache_purge (rd_kafka_t *rk) {
 
         if (!was_empty)
                 rd_kafka_metadata_cache_propagate_changes(rk);
+
+        if (purge_observers)
+                rd_list_clear(&rk->rk_metadata_cache.rkmc_observers);
 }
 
 
@@ -369,7 +372,7 @@ void rd_kafka_metadata_cache_update (rd_kafka_t *rk,
                      md->topic_cnt);
 
         if (abs_update)
-                rd_kafka_metadata_cache_purge(rk);
+                rd_kafka_metadata_cache_purge(rk, rd_false/*not observers*/);
 
 
         for (i = 0 ; i < md->topic_cnt ; i++)
@@ -443,7 +446,7 @@ void rd_kafka_metadata_cache_purge_hints (rd_kafka_t *rk,
  *
  * @returns the number of topic hints inserted.
  *
- * @locks rd_kafka_wrlock()
+ * @locks_required rd_kafka_wrlock()
  */
 int rd_kafka_metadata_cache_hint (rd_kafka_t *rk,
                                   const rd_list_t *topics, rd_list_t *dst,
@@ -494,6 +497,8 @@ int rd_kafka_metadata_cache_hint (rd_kafka_t *rk,
 /**
  * @brief Same as rd_kafka_metadata_cache_hint() but takes
  *        a topic+partition list as input instead.
+ *
+ * @locks_acquired rd_kafka_wrlock()
  */
 int rd_kafka_metadata_cache_hint_rktparlist (
         rd_kafka_t *rk,
@@ -506,9 +511,12 @@ int rd_kafka_metadata_cache_hint_rktparlist (
         rd_list_init(&topics, rktparlist->cnt, rd_free);
         rd_kafka_topic_partition_list_get_topic_names(rktparlist, &topics,
                                                       0/*dont include regex*/);
+        rd_kafka_wrlock(rk);
         r = rd_kafka_metadata_cache_hint(rk, &topics, dst,
                                          RD_KAFKA_RESP_ERR__WAIT_CACHE,
                                          replace);
+        rd_kafka_wrunlock(rk);
+
         rd_list_destroy(&topics);
         return r;
 }
@@ -540,15 +548,15 @@ void rd_kafka_metadata_cache_init (rd_kafka_t *rk) {
 }
 
 /**
- * @brief Purge and destroy metadata cache
+ * @brief Purge and destroy metadata cache.
  *
- * @locks rd_kafka_wrlock()
+ * @locks_required rd_kafka_wrlock()
  */
 void rd_kafka_metadata_cache_destroy (rd_kafka_t *rk) {
         rd_list_destroy(&rk->rk_metadata_cache.rkmc_observers);
         rd_kafka_timer_stop(&rk->rk_timers,
                             &rk->rk_metadata_cache.rkmc_query_tmr, 1/*lock*/);
-        rd_kafka_metadata_cache_purge(rk);
+        rd_kafka_metadata_cache_purge(rk, rd_true/*observers too*/);
         mtx_destroy(&rk->rk_metadata_cache.rkmc_full_lock);
         mtx_destroy(&rk->rk_metadata_cache.rkmc_cnd_lock);
         cnd_destroy(&rk->rk_metadata_cache.rkmc_cnd);
