@@ -290,7 +290,7 @@ rd_kafka_mock_committed_offset_find (const rd_kafka_mock_partition_t *mpart,
         const rd_kafka_mock_committed_offset_t *coff;
 
         TAILQ_FOREACH(coff, &mpart->committed_offsets, link) {
-                if (rd_kafkap_str_cmp_str(group, coff->group))
+                if (!rd_kafkap_str_cmp_str(group, coff->group))
                         return (rd_kafka_mock_committed_offset_t *)coff;
         }
 
@@ -672,7 +672,13 @@ static void rd_kafka_mock_connection_close (rd_kafka_mock_connection_t *mconn,
 void rd_kafka_mock_connection_send_response (rd_kafka_mock_connection_t *mconn,
                                              rd_kafka_buf_t *resp) {
 
-        resp->rkbuf_ts_sent = rd_clock();
+        if (resp->rkbuf_flags & RD_KAFKA_OP_F_FLEXVER) {
+                /* Empty struct tags */
+                rd_kafka_buf_write_i8(resp, 0);
+        }
+
+        /* rkbuf_ts_sent might be initialized with a RTT delay, else 0. */
+        resp->rkbuf_ts_sent += rd_clock();
 
         resp->rkbuf_reshdr.Size =
                 (int32_t)(rd_buf_write_pos(&resp->rkbuf_buf) - 4);
@@ -821,7 +827,6 @@ rd_kafka_mock_connection_read_request (rd_kafka_mock_connection_t *mconn,
         } else if (rd_buf_write_pos(&rkbuf->rkbuf_buf) -
                    RD_KAFKAP_REQHDR_SIZE == rkbuf->rkbuf_totlen) {
                 /* The full request is now read into the buffer. */
-                rd_kafkap_str_t clientid;
 
                 /* Set up response reader slice starting past the
                  * request header */
@@ -831,7 +836,10 @@ rd_kafka_mock_connection_read_request (rd_kafka_mock_connection_t *mconn,
                               RD_KAFKAP_REQHDR_SIZE);
 
                 /* For convenience, shave off the ClientId */
-                rd_kafka_buf_read_compact_str(rkbuf, &clientid);
+                rd_kafka_buf_skip_str(rkbuf);
+
+                /* And the flexible versions header tags, if any */
+                rd_kafka_buf_skip_tags(rkbuf);
 
                 /* Return the buffer to the caller */
                 *rkbufp = rkbuf;
@@ -858,6 +866,14 @@ rd_kafka_buf_t *rd_kafka_mock_buf_new_response (const rd_kafka_buf_t *request) {
         /* CorrId */
         rd_kafka_buf_write_i32(rkbuf, request->rkbuf_reqhdr.CorrId);
 
+        if (request->rkbuf_flags & RD_KAFKA_OP_F_FLEXVER) {
+                rkbuf->rkbuf_flags |= RD_KAFKA_OP_F_FLEXVER;
+                /* Write empty response header tags, unless this is the
+                 * ApiVersionResponse which needs to be backwards compatible. */
+                if (request->rkbuf_reqhdr.ApiKey != RD_KAFKAP_ApiVersion)
+                        rd_kafka_buf_write_i8(rkbuf, 0);
+        }
+
         return rkbuf;
 }
 
@@ -1460,20 +1476,20 @@ rd_kafka_mock_coord_set (rd_kafka_mock_cluster_t *mcluster,
  * @brief Remove and return the next error, or RD_KAFKA_RESP_ERR_NO_ERROR
  *        if no error.
  */
-static rd_kafka_resp_err_t
+static rd_kafka_mock_error_rtt_t
 rd_kafka_mock_error_stack_next (rd_kafka_mock_error_stack_t *errstack) {
-        rd_kafka_resp_err_t err;
+        rd_kafka_mock_error_rtt_t err_rtt = { RD_KAFKA_RESP_ERR_NO_ERROR, 0 };
 
         if (likely(errstack->cnt == 0))
-                return RD_KAFKA_RESP_ERR_NO_ERROR;
+                return err_rtt;
 
-        err = errstack->errs[0];
+        err_rtt = errstack->errs[0];
         errstack->cnt--;
         if (errstack->cnt > 0)
                 memmove(errstack->errs, &errstack->errs[1],
                         sizeof(*errstack->errs) * errstack->cnt);
 
-        return err;
+        return err_rtt;
 }
 
 
@@ -1516,40 +1532,80 @@ rd_kafka_mock_error_stack_get (rd_kafka_mock_error_stack_head_t *shead,
 
 
 /**
- * @brief Removes and returns the next request error for request type \p ApiKey.
+ * @brief Removes and returns the next request error for response's ApiKey.
+ *
+ * If the error stack has a corresponding rtt/delay it is set on the
+ * provided response \p resp buffer.
  */
 rd_kafka_resp_err_t
 rd_kafka_mock_next_request_error (rd_kafka_mock_connection_t *mconn,
-                                  int16_t ApiKey) {
+                                  rd_kafka_buf_t *resp) {
         rd_kafka_mock_cluster_t *mcluster = mconn->broker->cluster;
         rd_kafka_mock_error_stack_t *errstack;
-        rd_kafka_resp_err_t err;
+        rd_kafka_mock_error_rtt_t err_rtt;
 
         mtx_lock(&mcluster->lock);
 
         errstack = rd_kafka_mock_error_stack_find(&mconn->broker->errstacks,
-                                                  ApiKey);
+                                                  resp->rkbuf_reqhdr.ApiKey);
         if (likely(!errstack)) {
-                errstack = rd_kafka_mock_error_stack_find(&mcluster->errstacks,
-                                                          ApiKey);
+                errstack = rd_kafka_mock_error_stack_find(
+                        &mcluster->errstacks,
+                        resp->rkbuf_reqhdr.ApiKey);
                 if (likely(!errstack)) {
                         mtx_unlock(&mcluster->lock);
                         return RD_KAFKA_RESP_ERR_NO_ERROR;
                 }
         }
 
-        err = rd_kafka_mock_error_stack_next(errstack);
+        err_rtt = rd_kafka_mock_error_stack_next(errstack);
+        resp->rkbuf_ts_sent = err_rtt.rtt;
+
         mtx_unlock(&mcluster->lock);
 
-        return err;
+        /* If the error is ERR__TRANSPORT (a librdkafka-specific error code
+         * that will never be returned by a broker), we close the connection.
+         * This allows closing the connection as soon as a certain
+         * request is seen.
+         * The handler code in rdkafka_mock_handlers.c does not need to
+         * handle this case specifically and will generate a response and
+         * enqueue it, but the connection will be down by the time it will
+         * be sent.
+         * Note: Delayed disconnects (rtt-based) are not supported. */
+        if (err_rtt.err == RD_KAFKA_RESP_ERR__TRANSPORT) {
+                rd_kafka_dbg(mcluster->rk, MOCK, "MOCK",
+                             "Broker %"PRId32": Forcing close of connection "
+                             "from %s",
+                             mconn->broker->id,
+                             rd_sockaddr2str(&mconn->peer,
+                                             RD_SOCKADDR2STR_F_PORT));
+                rd_kafka_transport_shutdown(mconn->transport);
+        }
+
+
+        return err_rtt.err;
 }
 
 
+void rd_kafka_mock_clear_request_errors (rd_kafka_mock_cluster_t *mcluster,
+                                         int16_t ApiKey) {
+        rd_kafka_mock_error_stack_t *errstack;
+
+        mtx_lock(&mcluster->lock);
+
+        errstack = rd_kafka_mock_error_stack_find(&mcluster->errstacks, ApiKey);
+        if (errstack)
+                errstack->cnt = 0;
+
+        mtx_unlock(&mcluster->lock);
+}
 
 
-void rd_kafka_mock_push_request_errors (rd_kafka_mock_cluster_t *mcluster,
-                                        int16_t ApiKey, size_t cnt, ...) {
-        va_list ap;
+void
+rd_kafka_mock_push_request_errors_array (rd_kafka_mock_cluster_t *mcluster,
+                                         int16_t ApiKey,
+                                         size_t cnt,
+                                         const rd_kafka_resp_err_t *errors) {
         rd_kafka_mock_error_stack_t *errstack;
         size_t totcnt;
 
@@ -1566,20 +1622,32 @@ void rd_kafka_mock_push_request_errors (rd_kafka_mock_cluster_t *mcluster,
                                             sizeof(*errstack->errs));
         }
 
-        va_start(ap, cnt);
-        while (cnt-- > 0)
-                errstack->errs[errstack->cnt++] =
-                        va_arg(ap, rd_kafka_resp_err_t);
-        va_end(ap);
+        while (cnt > 0) {
+                errstack->errs[errstack->cnt].err = errors[--cnt];
+                errstack->errs[errstack->cnt++].rtt = 0;
+        }
 
         mtx_unlock(&mcluster->lock);
 }
 
+void rd_kafka_mock_push_request_errors (rd_kafka_mock_cluster_t *mcluster,
+                                        int16_t ApiKey, size_t cnt, ...) {
+        va_list ap;
+        rd_kafka_resp_err_t *errors = rd_alloca(sizeof(*errors) * cnt);
+        size_t i;
+
+        va_start(ap, cnt);
+        for (i = 0 ; i < cnt ; i++)
+                errors[i] = va_arg(ap, rd_kafka_resp_err_t);
+
+        rd_kafka_mock_push_request_errors_array(mcluster, ApiKey, cnt, errors);
+}
+
 
 rd_kafka_resp_err_t
-rd_kafka_mock_broker_push_request_errors (rd_kafka_mock_cluster_t *mcluster,
-                                          int32_t broker_id,
-                                          int16_t ApiKey, size_t cnt, ...) {
+rd_kafka_mock_broker_push_request_error_rtts (rd_kafka_mock_cluster_t *mcluster,
+                                              int32_t broker_id,
+                                              int16_t ApiKey, size_t cnt, ...) {
         rd_kafka_mock_broker_t *mrkb;
         va_list ap;
         rd_kafka_mock_error_stack_t *errstack;
@@ -1604,9 +1672,12 @@ rd_kafka_mock_broker_push_request_errors (rd_kafka_mock_cluster_t *mcluster,
         }
 
         va_start(ap, cnt);
-        while (cnt-- > 0)
-                errstack->errs[errstack->cnt++] =
+        while (cnt-- > 0) {
+                errstack->errs[errstack->cnt].err =
                         va_arg(ap, rd_kafka_resp_err_t);
+                errstack->errs[errstack->cnt++].rtt =
+                        ((rd_ts_t)va_arg(ap, int)) * 1000;
+        }
         va_end(ap);
 
         mtx_unlock(&mcluster->lock);
@@ -2002,6 +2073,8 @@ rd_kafka_mock_cluster_destroy0 (rd_kafka_mock_cluster_t *mcluster) {
         while ((mcoord = TAILQ_FIRST(&mcluster->coords)))
                 rd_kafka_mock_coord_destroy(mcluster, mcoord);
 
+        rd_list_destroy(&mcluster->pids);
+
         while ((errstack = TAILQ_FIRST(&mcluster->errstacks))) {
                 TAILQ_REMOVE(&mcluster->errstacks, errstack, link);
                 rd_kafka_mock_error_stack_destroy(errstack);
@@ -2073,14 +2146,12 @@ rd_kafka_mock_cluster_t *rd_kafka_mock_cluster_new (rd_kafka_t *rk,
         mcluster = rd_calloc(1, sizeof(*mcluster));
         mcluster->rk = rk;
 
-        rd_kafka_timers_init(&mcluster->timers, rk);
-
         mcluster->dummy_rkb = rd_kafka_broker_add(rk, RD_KAFKA_INTERNAL,
                                                   RD_KAFKA_PROTO_PLAINTEXT,
                                                   "mock", 0,
                                                   RD_KAFKA_NODEID_UA);
         rd_snprintf(mcluster->id, sizeof(mcluster->id),
-                    "mockCluster%lx", (intptr_t)rk ^ (intptr_t)mcluster);
+                    "mockCluster%lx", (intptr_t)mcluster >> 2);
 
         TAILQ_INIT(&mcluster->brokers);
 
@@ -2104,6 +2175,8 @@ rd_kafka_mock_cluster_t *rd_kafka_mock_cluster_new (rd_kafka_t *rk,
 
         TAILQ_INIT(&mcluster->coords);
 
+        rd_list_init(&mcluster->pids, 16, rd_free);
+
         TAILQ_INIT(&mcluster->errstacks);
 
         memcpy(mcluster->api_handlers, rd_kafka_mock_api_handlers,
@@ -2115,6 +2188,7 @@ rd_kafka_mock_cluster_t *rd_kafka_mock_cluster_new (rd_kafka_t *rk,
         mcluster->ops->rkq_serve = rd_kafka_mock_cluster_op_serve;
         mcluster->ops->rkq_opaque = mcluster;
 
+        rd_kafka_timers_init(&mcluster->timers, rk, mcluster->ops);
 
         if ((r = rd_pipe_nonblocking(mcluster->wakeup_fds)) == -1) {
                 rd_kafka_log(rk, LOG_ERR, "MOCK",
