@@ -95,6 +95,7 @@ const char *rd_kafka_op2str (rd_kafka_op_type_t type) {
                 [RD_KAFKA_OP_GET_REBALANCE_PROTOCOL] =
                 "REPLY:GET_REBALANCE_PROTOCOL",
                 [RD_KAFKA_OP_LEADERS] = "REPLY:LEADERS",
+                [RD_KAFKA_OP_BARRIER] = "REPLY:BARRIER",
         };
 
         if (type & RD_KAFKA_OP_REPLY)
@@ -237,6 +238,7 @@ rd_kafka_op_t *rd_kafka_op_new0 (const char *source, rd_kafka_op_type_t type) {
                 [RD_KAFKA_OP_GET_REBALANCE_PROTOCOL] =
                 sizeof(rko->rko_u.rebalance_protocol),
                 [RD_KAFKA_OP_LEADERS] = sizeof(rko->rko_u.leaders),
+                [RD_KAFKA_OP_BARRIER] = _RD_KAFKA_OP_EMPTY,
         };
         size_t tsize = op2size[type & ~RD_KAFKA_OP_FLAGMASK];
 
@@ -468,8 +470,8 @@ void rd_kafka_q_op_err (rd_kafka_q_t *rkq, rd_kafka_resp_err_t err,
  *                  if not applicable.
  * @param err Error code.
  * @param version Queue version barrier, or 0 if not applicable.
- * @param topic May be NULL. Mutually exclusive with \p rktp.
- * @param rktp May be NULL. Mutually exclusive with \p topic.
+ * @param topic May be NULL.
+ * @param rktp May be NULL. Takes precedence over \p topic.
  * @param offset RD_KAFKA_OFFSET_INVALID if not applicable.
  *
  * @sa rd_kafka_q_op_err()
@@ -493,10 +495,9 @@ void rd_kafka_consumer_err (rd_kafka_q_t *rkq, int32_t broker_id,
         rko->rko_u.err.errstr = rd_strdup(buf);
         rko->rko_u.err.rkm.rkm_broker_id = broker_id;
 
-        if (rktp) {
-                rd_assert(!topic);
+        if (rktp)
                 rko->rko_rktp = rd_kafka_toppar_keep(rktp);
-        } else if (topic)
+        else if (topic)
                 rko->rko_u.err.rkm.rkm_rkmessage.rkt =
                         (rd_kafka_topic_t *)rd_kafka_lwtopic_new(rkq->rkq_rk,
                                                                  topic);
@@ -778,7 +779,8 @@ void rd_kafka_op_throttle_time (rd_kafka_broker_t *rkb,
 				int throttle_time) {
 	rd_kafka_op_t *rko;
 
-	rd_avg_add(&rkb->rkb_avg_throttle, throttle_time);
+        if (unlikely(throttle_time > 0))
+                rd_avg_add(&rkb->rkb_avg_throttle, throttle_time);
 
 	/* We send throttle events when:
 	 *  - throttle_time > 0
