@@ -55,6 +55,8 @@ int rd_kafka_err_action (rd_kafka_broker_t *rkb,
 			 const rd_kafka_buf_t *request, ...);
 
 
+const char *rd_kafka_actions2str (int actions);
+
 rd_kafka_topic_partition_list_t *
 rd_kafka_buf_read_topic_partitions (rd_kafka_buf_t *rkbuf,
                                     size_t estimated_part_cnt,
@@ -77,20 +79,20 @@ rd_kafka_FindCoordinatorRequest (rd_kafka_broker_t *rkb,
                                  rd_kafka_resp_cb_t *resp_cb,
                                  void *opaque);
 
-rd_kafka_resp_err_t rd_kafka_handle_Offset (rd_kafka_t *rk,
-					    rd_kafka_broker_t *rkb,
-					    rd_kafka_resp_err_t err,
-					    rd_kafka_buf_t *rkbuf,
-					    rd_kafka_buf_t *request,
-                                            rd_kafka_topic_partition_list_t
-                                            *offsets);
+rd_kafka_resp_err_t rd_kafka_handle_ListOffsets (rd_kafka_t *rk,
+                                                 rd_kafka_broker_t *rkb,
+                                                 rd_kafka_resp_err_t err,
+                                                 rd_kafka_buf_t *rkbuf,
+                                                 rd_kafka_buf_t *request,
+                                                 rd_kafka_topic_partition_list_t
+                                                 *offsets,
+                                                 int *actionsp);
 
-void rd_kafka_OffsetRequest (rd_kafka_broker_t *rkb,
-                             rd_kafka_topic_partition_list_t *offsets,
-                             int16_t api_version,
-                             rd_kafka_replyq_t replyq,
-                             rd_kafka_resp_cb_t *resp_cb,
-                             void *opaque);
+void rd_kafka_ListOffsetsRequest (rd_kafka_broker_t *rkb,
+                                  rd_kafka_topic_partition_list_t *offsets,
+                                  rd_kafka_replyq_t replyq,
+                                  rd_kafka_resp_cb_t *resp_cb,
+                                  void *opaque);
 
 rd_kafka_resp_err_t
 rd_kafka_handle_OffsetFetch (rd_kafka_t *rk,
@@ -100,7 +102,8 @@ rd_kafka_handle_OffsetFetch (rd_kafka_t *rk,
                              rd_kafka_buf_t *request,
                              rd_kafka_topic_partition_list_t **offsets,
                              rd_bool_t update_toppar,
-                             rd_bool_t add_part);
+                             rd_bool_t add_part,
+                             rd_bool_t allow_retry);
 
 void rd_kafka_op_handle_OffsetFetch (rd_kafka_t *rk,
 				     rd_kafka_broker_t *rkb,
@@ -247,7 +250,8 @@ void rd_kafka_SaslAuthenticateRequest (rd_kafka_broker_t *rkb,
                                        void *opaque);
 
 int rd_kafka_ProduceRequest (rd_kafka_broker_t *rkb, rd_kafka_toppar_t *rktp,
-                             const rd_kafka_pid_t pid);
+                             const rd_kafka_pid_t pid,
+                             uint64_t epoch_base_msgid);
 
 rd_kafka_resp_err_t
 rd_kafka_CreateTopicsRequest (rd_kafka_broker_t *rkb,
