@@ -104,6 +104,11 @@ struct rd_kafka_topic_s {
 	rd_kafka_toppar_t **rkt_p;          /**< Partition array */
 	int32_t            rkt_partition_cnt;
 
+        int32_t            rkt_sticky_partition;    /**< Current sticky partition.
+                                                     *     @locks rkt_lock */
+        rd_interval_t      rkt_sticky_intvl;        /**< Interval to assign new 
+                                                     *   sticky partition. */
+
         rd_list_t          rkt_desp;              /* Desired partitions
                                                    * that are not yet seen
                                                    * in the cluster. */
@@ -230,7 +235,7 @@ typedef struct rd_kafka_topic_info_s {
 	int   partition_cnt;
 } rd_kafka_topic_info_t;
 
-
+int rd_kafka_topic_info_topic_cmp (const void *_a, const void *_b);
 int rd_kafka_topic_info_cmp (const void *_a, const void *_b);
 rd_kafka_topic_info_t *rd_kafka_topic_info_new (const char *topic,
 						int partition_cnt);
@@ -256,7 +261,8 @@ void rd_kafka_topic_leader_query0 (rd_kafka_t *rk, rd_kafka_topic_t *rkt,
 #define rd_kafka_topic_fast_leader_query(rk) \
         rd_kafka_metadata_fast_leader_query(rk)
 
-void rd_kafka_local_topics_to_list (rd_kafka_t *rk, rd_list_t *topics);
+void rd_kafka_local_topics_to_list (rd_kafka_t *rk, rd_list_t *topics,
+                                    int *cache_cntp);
 
 void rd_ut_kafka_topic_set_topic_exists (rd_kafka_topic_t *rkt,
                                          int partition_cnt,
