@@ -116,7 +116,6 @@ static void do_test_bsearch (void) {
 
   topic = Test::mk_topic_name("0059-bsearch", 1);
   Test::conf_init(&conf, &tconf, 0);
-  Test::conf_set(tconf, "produce.offset.report", "true");
   Test::conf_set(tconf, "acks", "all");
   Test::conf_set(conf, "api.version.request", "true");
   conf->set("dr_cb", &my_dr, errstr);
@@ -160,7 +159,6 @@ static void do_test_bsearch (void) {
   Test::conf_set(conf, "fetch.error.backoff.ms", "1");
   Test::conf_set(conf, "queued.min.messages", "1");
   Test::conf_set(conf, "enable.auto.commit", "false");
-  Test::conf_set(conf, "socket.blocking.max.ms", "2000");
 
   RdKafka::KafkaConsumer *c = RdKafka::KafkaConsumer::create(conf, errstr);
   if (!c)
