@@ -237,7 +237,7 @@ static RD_INLINE int64_t test_clock (void) {
 
 
 typedef struct test_timing_s {
-        char name[400];
+        char name[450];
         int64_t ts_start;
         int64_t duration;
         int64_t ts_every; /* Last every */
@@ -321,15 +321,21 @@ static RD_UNUSED int TIMING_EVERY (test_timing_t *timing, int us) {
 int test_sub_start (const char *func, int line, int is_quick,
                     const char *fmt, ...);
 void test_sub_pass (void);
+void test_sub_skip (const char *fmt, ...);
+
 #define SUB_TEST0(IS_QUICK,...) do {                                    \
                 if (!test_sub_start(__FUNCTION__, __LINE__,             \
                                     IS_QUICK, __VA_ARGS__))             \
                         return;                                         \
         } while (0)
 
-#define SUB_TEST(...) SUB_TEST0(rd_false, "" __VA_ARGS__)
-#define SUB_TEST_QUICK(...) SUB_TEST0(rd_true, "" __VA_ARGS__)
+#define SUB_TEST(...) SUB_TEST0(0, "" __VA_ARGS__)
+#define SUB_TEST_QUICK(...) SUB_TEST0(1, "" __VA_ARGS__)
 #define SUB_TEST_PASS() test_sub_pass()
+#define SUB_TEST_SKIP(...) do {                 \
+                test_sub_skip(__VA_ARGS__);     \
+                return;                         \
+        } while (0)
 
 
 #ifndef _WIN32
